@@ -237,7 +237,7 @@ int comex_free_local(void *ptr)
 }
 
 
-int comex_init()
+int _comex_init(MPI_Comm comm)
 {
     int init_flag;
     
@@ -252,7 +252,7 @@ int comex_init()
     assert(init_flag);
     
     /* Duplicate the World Communicator */
-    MPI_Comm_dup(MPI_COMM_WORLD, &(l_state.world_comm));
+    MPI_Comm_dup(comm, &(l_state.world_comm));
    
     /* My Rank */
     MPI_Comm_rank(l_state.world_comm, &(l_state.rank));
@@ -279,6 +279,16 @@ int comex_init()
 
 }
 
+int comex_init()
+{
+  return _comex_init(MPI_COMM_WORLD);
+}
+
+int comex_init_comm(MPI_Comm comm)
+{
+  return _comex_init(comm);
+}
+
 int comex_init_args(int *argc, char ***argv)
 {
     int rc;
