@@ -6,10 +6,14 @@
 #include "gacommon.h"
 #include "typesf2c.h"
 
+#include <mpi.h>
+
 typedef intp AccessIndex;
 
 /* Routines from base.c */
+extern void pnga_version(Integer *major, Integer *minor, Integer *patch);
 extern logical pnga_allocate(Integer g_a);
+extern logical pnga_deallocate(Integer g_a);
 extern logical pnga_compare_distr(Integer g_a, Integer g_b);
 extern logical pnga_create(Integer type, Integer ndim,
                            Integer *dims, char* name,
@@ -51,14 +55,17 @@ extern logical pnga_destroy_mutexes();
 extern void pnga_distribution(Integer g_a, Integer proc, Integer *lo, Integer *hi);
 extern logical pnga_duplicate(Integer g_a, Integer *g_b, char *array_name);
 extern void pnga_fill(Integer g_a, void* val);
+extern void pnga_free(void* ptr);
 extern void pnga_get_block_info(Integer g_a, Integer *num_blocks,
                                 Integer *block_dims);
 extern logical pnga_get_debug();
 extern Integer pnga_get_dimension(Integer g_a);
+extern void pnga_get_distribution_type(Integer g_a, char *type);
 extern void pnga_get_proc_grid(Integer g_a, Integer *dims);
 extern void pnga_get_proc_index(Integer g_a, Integer iproc, Integer *index);
 extern logical pnga_has_ghosts(Integer g_a);
 extern void pnga_initialize();
+extern int pnga_initialize_comm(MPI_Comm comm);
 extern int  pnga_initialized();
 extern void pnga_initialize_ltd(Integer limit);
 extern void pnga_inquire(Integer g_a, Integer *type, Integer *ndim, Integer *dims);
@@ -73,7 +80,7 @@ extern logical pnga_locate_nnodes(Integer g_a, Integer *lo, Integer *hi, Integer
 extern logical pnga_locate_region(Integer g_a, Integer *lo, Integer *hi,
                                   Integer *map, Integer *proclist, Integer *np);
 extern void pnga_lock(Integer mutex);
-extern Integer pnga_ndim(Integer g_a);
+extern void* pnga_malloc(Integer nelem, int type, char *name);
 extern void pnga_mask_sync(Integer begin, Integer end);
 extern Integer pnga_memory_avail();
 extern logical pnga_memory_limited();
@@ -82,10 +89,14 @@ extern void pnga_merge_distr_patch(Integer g_a, Integer *alo, Integer *ahi,
 extern void pnga_merge_mirrored(Integer g_a);
 extern void pnga_nblock(Integer g_a, Integer *nblock);
 
+extern Integer pnga_ndim(Integer g_a);
 extern Integer pnga_nnodes();
 extern Integer pnga_nodeid();
+extern logical pnga_overlay(Integer g_a, Integer g_p);
 extern Integer pnga_pgroup_absolute_id(Integer grp, Integer pid);
 extern Integer pnga_pgroup_create(Integer *list, Integer count);
+extern Integer pnga_pgroup_duplicate(Integer grp);
+extern Integer pnga_pgroup_self();
 extern logical pnga_pgroup_destroy(Integer grp);
 extern Integer pnga_pgroup_get_default();
 extern Integer pnga_pgroup_get_mirror();
@@ -100,9 +111,11 @@ extern void pnga_randomize(Integer g_a, void* val);
 extern Integer pnga_get_pgroup(Integer g_a);
 extern Integer pnga_get_pgroup_size(Integer grp_id);
 extern void pnga_set_array_name(Integer g_a, char *array_name);
+extern void pnga_get_array_name(Integer g_a, char *array_name);
 extern void pnga_set_block_cyclic(Integer g_a, Integer *dims);
 extern void pnga_set_block_cyclic_proc_grid(Integer g_a, Integer *dims, Integer *proc_grid);
 extern void pnga_set_tiled_proc_grid(Integer g_a, Integer *dims, Integer *proc_grid);
+extern void pnga_set_tiled_irreg_proc_grid(Integer g_a, Integer *mapc, Integer *nblocks, Integer *proc_grid);
 extern void pnga_set_chunk(Integer g_a, Integer *chunk);
 extern void pnga_set_data(Integer g_a, Integer ndim, Integer *dims, Integer type);
 extern void pnga_set_debug(logical flag);
@@ -115,11 +128,14 @@ extern void pnga_set_restricted(Integer g_a, Integer *list, Integer size);
 extern void pnga_set_restricted_range(Integer g_a, Integer lo_proc, Integer hi_proc);
 extern void pnga_set_property(Integer g_a, char *property);
 extern void pnga_unset_property(Integer g_a);
+extern void pnga_set_memory_dev(Integer g_a, char *device);
 extern void pnga_terminate();
 extern Integer pnga_total_blocks(Integer g_a);
 extern void pnga_unlock(Integer mutex);
 extern logical pnga_uses_ma();
 extern logical pnga_uses_proc_grid(Integer g_a);
+extern logical pnga_uses_irreg_proc_grid(Integer g_a);
+extern void pnga_get_map_info(Integer g_a, Integer *num_blocks, Integer **map);
 extern logical pnga_valid_handle(Integer g_a);
 extern Integer pnga_verify_handle(Integer g_a);
 extern void pnga_check_handle(Integer g_a, char *string);
@@ -190,6 +206,7 @@ extern void pnga_strided_put(Integer g_a, Integer *lo, Integer *hi, Integer *ski
                              void *buf, Integer *ld);
 extern void pnga_sync();
 extern DoublePrecision pnga_wtime();
+extern DoublePrecision pnga_rand(Integer idum);
 
 /* Routines from datatypes.c */
 extern Integer pnga_type_f2c(Integer type);
@@ -340,6 +357,8 @@ extern void pnga_periodic(Integer g_a, Integer *lo, Integer *hi, void *buf, Inte
 extern void pnga_matmul(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer ailo, Integer aihi, Integer ajlo, Integer ajhi, Integer g_b, Integer bilo, Integer bihi, Integer bjlo, Integer bjhi, Integer g_c, Integer cilo, Integer cihi, Integer cjlo, Integer cjhi);
 extern void pnga_matmul_mirrored(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer ailo, Integer aihi, Integer ajlo, Integer ajhi, Integer g_b, Integer bilo, Integer bihi, Integer bjlo, Integer bjhi, Integer g_c, Integer cilo, Integer cihi, Integer cjlo, Integer cjhi);
 extern void pnga_matmul_patch(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer alo[], Integer ahi[], Integer g_b, Integer blo[], Integer bhi[], Integer g_c, Integer clo[], Integer chi[]);
+extern void pnga_matmul_patch_alt(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer alo[], Integer ahi[], Integer g_b, Integer blo[], Integer bhi[], Integer g_c, Integer clo[], Integer chi[]);
+extern void pnga_matmul_basic(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer alo[], Integer ahi[], Integer g_b, Integer blo[], Integer bhi[], Integer g_c, Integer clo[], Integer chi[]);
 extern void pnga_matmul_basic(char *transa, char *transb, void *alpha, void *beta, Integer g_a, Integer alo[], Integer ahi[], Integer g_b, Integer blo[], Integer bhi[], Integer g_c, Integer clo[], Integer chi[]);
 
 /* Routines from ga_diag_seqc.c */
@@ -382,4 +401,34 @@ extern void pnga_nbget_field(Integer g_a, Integer *lo, Integer *hi, Integer foff
 extern void pnga_nbput_field(Integer g_a, Integer *lo, Integer *hi, Integer foff, Integer fsize,void *buf, Integer *ld, Integer *nbhandle);
 extern void pnga_put_field(Integer g_a, Integer *lo, Integer *hi, Integer foff, Integer fsize,void *buf, Integer *ld);
 
+
+/* Routines for sparse matrices */
+
+extern Integer pnga_sprs_array_create(Integer idim, Integer jdim, Integer type, Integer trans);
+extern Integer pnga_sprs_array_create_from_dense(Integer g_a, Integer idxsize, Integer flag);
+extern Integer pnga_sprs_array_create_from_sparse(Integer s_a, Integer flag);
+extern void pnga_sprs_array_add_element(Integer s_a, Integer idx, Integer jdx, void *val);
+extern logical pnga_sprs_array_assemble(Integer s_a);
+extern void pnga_sprs_array_row_distribution(Integer s_a, Integer iproc, Integer *lo, Integer *hi);
+extern void pnga_sprs_array_column_distribution(Integer s_a, Integer iproc, Integer *lo, Integer *hi);
+extern void pnga_sprs_array_access_col_block(Integer s_a, Integer icol, void *idx, void *jdx, void *val);
+extern int pnga_sprs_array_access_col_block_idx(Integer s_a, Integer icol, AccessIndex *idx, AccessIndex *jdx, AccessIndex *vdx);
+extern void pnga_sprs_array_col_block_list(Integer s_a, Integer **idx, Integer *n);
+extern Integer pnga_sprs_array_duplicate(Integer s_a);
+extern Integer pnga_sprs_array_matmat_multiply(Integer s_a, Integer s_b);
+extern Integer pnga_sprs_array_sprsdns_multiply(Integer s_a, Integer g_b, Integer trans);
+extern Integer pnga_sprs_array_dnssprs_multiply(Integer g_a, Integer s_b, Integer trans);
+extern Integer pnga_sprs_array_count_sketch(Integer s_a, Integer size_k,
+    Integer *g_k, Integer *g_w, Integer trans);
+extern void pnga_sprs_array_matvec_multiply(Integer s_a, Integer g_a, Integer g_v);
+extern logical pnga_sprs_array_destroy(Integer s_a);
+extern void pnga_sprs_array_export(Integer s_a, const char* file);
+extern void pnga_sprs_array_get_diag(Integer s_a, Integer *g_d);
+extern void pnga_sprs_array_diag_right_multiply(Integer s_a, Integer g_d);
+extern void pnga_sprs_array_diag_left_multiply(Integer s_a, Integer g_d);
+extern void pnga_sprs_array_shift_diag(Integer s_a, void *shift);
+extern logical pnga_sprs_array_get_block(Integer s_a, Integer irow, Integer icol,
+    void **idx, void **jdx, void **data, Integer *ilo, Integer *ihi,
+    Integer *jlo, Integer *jhi);
+extern Integer pnga_sprs_array_get_column(Integer g_v, Integer irow);
 #endif /* PAPI_H_ */
