@@ -15,7 +15,7 @@
 #define DIM 2
 #define GSIZE 5
 
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
   int rank, nprocs, i, j;
   int g_A, g_B, g_V, local_A[5][5], local_B[5][5], vsize=GSIZE;  
