@@ -48,6 +48,10 @@
 #   include <unistd.h>
 #endif
 
+#ifndef PATH_MAX
+#   define PATH_MAX 4096
+#endif
+
 #ifdef EAF_STATS
 #   include <sys/types.h>
 #   include <sys/time.h>
@@ -64,6 +68,7 @@
 #   define EAF_MAX_FILES 1024
 #endif
 
+static int eafhack_openfiles=0;
 
 static struct {
     char *fname;      /**< Filename --- if non-null is active*/
@@ -138,10 +143,6 @@ int EAF_Open(const char *fname, int type, int *fd)
   int i=0, j=0, found=0;
   char *ptr;
   long handle, index;
-#ifdef CRAY_XT
-    int myid;
-#   include <mpi.h>
-#endif
     while ((i<EAF_MAX_FILES) && file[i].fname) /* Find first empty slot */
         i++;
     if (i == EAF_MAX_FILES) return EAF_ERR_MAX_OPEN;
@@ -153,23 +154,24 @@ int EAF_Open(const char *fname, int type, int *fd)
 	break;
       }
     }
+#ifdef DEBUG
+    printf(" JJJ %d III %d fname %s filejfname %s found %d type %d\n", j, i, fname, file[j].fname, found, type);
+#endif
       
     if(type > 0) {
       /* check if this file aka MA region labeled by fname is already open with size >=0*/
 #ifdef DEBUG
-      printf(" JJJ %d III %d fname %s filejfname %s found %d \n", j, i, fname, file[j].fname, found);
+      printf(" fname %s type %d found %d \n", fname, type, found);
 #endif
       if(found == 0 ) {
-/* if arg gt 1M then use remainder as size */
-	/* we grab 3/4 of avail mem */
-        if(type > 1000000) {
-	  file[i].size=type-1000000;
-	}else{
-	file[i].size=MA_inquire_avail(MT_CHAR)*8/10;
-	}
+	file[i].size=type*1024*1024;
+#ifdef DEBUG
+	printf("  fname %s type %lf alloc_get size %lf \n", fname, type, file[i].size);
+#endif
 
 	if (!MA_alloc_get(MT_CHAR, file[i].size, fname, &handle, &index))
 	  return EAF_ERR_OPEN;
+	if ((i +1) > eafhack_openfiles) eafhack_openfiles++;
     /* MA hack: we pass   type = sizeof MA alloc in megabytes */
 	MA_get_pointer(handle, &ptr);
     if (!(file[i].fname = strdup(fname)))
@@ -200,18 +202,9 @@ int EAF_Open(const char *fname, int type, int *fd)
 #endif
 
       if (!(file[i].elio_fd = elio_open(fname, type, ELIO_PRIVATE))) {
-#ifdef CRAY_XT
-        MPI_Comm_rank(MPI_COMM_WORLD,&myid);
-        /* printf(" %d sleeping for %d usec \n", myid, (myid+1)/4); */
-        usleep((myid+1)/4);
-        if (!(file[i].elio_fd = elio_open(fname, type, ELIO_PRIVATE))) {
-#endif
             free(file[i].fname);
             file[i].fname = 0;
             return ELIO_PENDING_ERR;
-#ifdef CRAY_XT
-        }
-#endif
       }
     }
 
@@ -467,15 +460,17 @@ int EAF_Delete(const char *fname)
 
   int  j, found=0;
   /* get fd from fname */
-  for (j=0; (j< EAF_MAX_FILES) && file[j].fname; j++){
+  for (j=0; (j< eafhack_openfiles); j++){
+    if(file[j].fname){
       if(strcmp(file[j].fname,fname) == 0 && file[j].size >0) {
 	found=1;
 	break;
       }
     }
+    }
 #ifdef DEBUG
   printf("eaf_delete: fname %s found %d \n", fname, found);
-  if (found ==1) printf("eaf_delete: j %d filej.fname %s \n", j, file[j].fname);
+  if (found ==1) printf("eaf_delete: j %d filej.fname %s handle %d \n", j, file[j].fname,file[j].handle);
 #endif
     if (found > 0) {
        if(!MA_free_heap(file[j].handle)) {
@@ -506,7 +501,7 @@ int EAF_Delete(const char *fname)
  *
  * Return 0 on success, non-zero on failure.
  */
-int EAF_Stat(const char *path, int *avail_mb, char *fstype, int fslen)
+int EAF_Stat(const char *path, long *avail_mb, char *fstype, int fslen)
 {
     char dirname[PATH_MAX];
     stat_t statinfo;
@@ -516,7 +511,7 @@ int EAF_Stat(const char *path, int *avail_mb, char *fstype, int fslen)
     if ((rc = elio_stat(dirname, &statinfo))) return rc;
     if (fslen < 8) return EAF_ERR_TOO_SHORT;
 
-    *avail_mb = (int)(statinfo.avail>>10);
+    *avail_mb = (long)(statinfo.avail>>10);
     if (statinfo.fs == ELIO_UFS)
         strcpy(fstype, "UFS");
     else if (statinfo.fs == ELIO_PFS)
@@ -597,28 +592,9 @@ void EAF_Errmsg(int code, char *msg)
  */
 int EAF_Truncate(int fd, eaf_off_t length)
 {
-#ifdef CRAY 
-    int rc;
-#endif
-
     if (!valid_fd(fd)) return EAF_ERR_INVALID_FD;
 
-#ifdef CRAY 
-    /* ftruncate does not work with Cray FFIO, we need to implement it
-     * as a sequence of generic close, truncate, open calls 
-     */
-
-    rc = elio_close(file[fd].elio_fd);
-    if(rc) return rc;
-    if(truncate(file[fd].fname, (off_t) length)) return EAF_ERR_TRUNCATE;  
-    if (!(file[fd].elio_fd = elio_open(file[fd].fname, file[fd].type, ELIO_PRIVATE))) {
-        free(file[fd].fname);
-        file[fd].fname = 0;
-        return ELIO_PENDING_ERR;
-    }
-#else
     if(elio_truncate(file[fd].elio_fd, (Off_t)length)) return EAF_ERR_TRUNCATE;
-#endif
 
     return EAF_OK;
     /*  return elio_truncate(file[fd].elio_fd, (Off_t) length);*/
